--[[
 AutoSummon module for MiniPet
 Version 5.1.247
 $Revision: 183 $

 All credit for authoring this addon is listed below
]]--

--[[
Copyright (c) 2010-2012, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "MiniPet autosummon Module" )

local SpecialEvents_Aura = LibStub( "LibLordFarlander-SpecialEvents-Aura-4.0")
if( not SpecialEvents_Aura ) then
    return
end--if

local PetAndMountDatabase = LibStub( "LibLordFarlander-PetAndMountDatabase-1.1", true )
if( not PetAndMountDatabase ) then
    return
end--if

if( not LibStub( "LibLordFarlander-Randomizer-2.0", true ) ) then
    return
end--if

local moduleName = "AutoSummon"

local AutoSummon = {
    displayname = L["Auto Summon"],
    settings = {
        [L["autosummon"]] = {
            name = L["Auto summon"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically summoned any time they are not out."],
                VariableName = "AutoSummon",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["combatWait"]] = {
            name = L["Combat wait"],
            type = "toggle",
            arg = {
                DescriptionString = L["Toggles whether to wait until after combat is over before automatically summoning."],
                VariableName = "CombatWait",
                Default = true,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["pvpSummon"]] = {
            type = "toggle",
            name = L["Summon in PvP areas"],
            arg = {
                DescriptionString = L["Toggles whether companions will be automatically summoned when in a PvP area (battleground, arena, etc.)."],
                VariableName = "PvPSummon",
                Default = false,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
        [L["unsummonWhenStealthed"]] = {
            name = L["Unsummon when stealthed"],
            type = "toggle",
            width = "double",
            arg = {
                DescriptionString = L["Toggles whether companions that are learned should be automatically unsummoned when stealthing/shadowmelding/etc."],
                VariableName = "UnsummonWhenStealthed",
                Default = true,
            },
            desc = MiniPet.BooleanOptionDescription,
            get = "GetGenericOption",
            set = "SetGenericOption",
        },
    },
    events = {
        "Update",
        "LOOT_CLOSED",
        "LOOT_OPENED",
        "UNIT_SPELLCAST_CHANNEL_START",
        "UNIT_SPELLCAST_CHANNEL_STOP",
        "UNIT_SPELLCAST_FAILED",
        "UNIT_SPELLCAST_FAILED_QUIET",
        "UNIT_SPELLCAST_INTERRUPTED",
        "UNIT_SPELLCAST_SENT",
        "UNIT_SPELLCAST_START",
        "UNIT_SPELLCAST_STOP",
        "UNIT_SPELLCAST_SUCCEEDED",
        "UPDATE_STEALTH",
    },
    -- Module variables
    isChanneling = false,
    isLooting = false,
    delaySummon = false,
    counter = 0,
}

function AutoSummon:OnUPDATE_STEALTH( MiniPet )
    if( IsStealthed() ) then
        if( MiniPet:GetSetting( moduleName, "profile", "UnsummonWhenStealthed" ) and
            MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) )  then
            MiniPet:DoCustomEvent( "DismissCompanion" )
            self.delaySummon = false
        end--if
    end--if
end--AutoSummon:OnUPDATE_STEALTH( MiniPet )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SENT( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = true
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_START( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_INTERRUPTED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_STOP(( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_FAILED_QUIET( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_SUCCEEDED( MiniPet, unit )

function AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )
    if( unit == "player" ) then
        self.isChanneling = false
    end--if
end--AutoSummon:OnUNIT_SPELLCAST_CHANNEL_STOP( MiniPet, unit )

function AutoSummon:OnLOOT_OPENED( MiniPet )
    self.isLooting = true
end--AutoSummon:OnLOOT_OPENED( MiniPet )

function AutoSummon:OnLOOT_CLOSED( MiniPet )
    self.isLooting = false
end--AutoSummon:OnLOOT_CLOSED( MiniPet )

function AutoSummon:OnUpdate( MiniPet )
    if( ( not( self.IsInPvPZone() and MiniPet:GetSetting( moduleName, "profile", "PvPSummon" ) ) ) and
        ( ( self.delaySummon and MiniPet:GetSetting( moduleName, "profile", "AutoSummon" ) ) and
          not( MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) ) ) ) then
        if( not( self.isChanneling ) and not( self.isLooting ) and not( self.IsStealthed() ) and
            not( IsMounted() ) and not( UnitIsDeadOrGhost( "player" ) ) and
            not( UnitInVehicle( "player" ) ) and not( UnitOnTaxi( "player" ) ) and
            not( InCombatLockdown() and MiniPet:GetSetting( moduleName, "profile", "CombatWait" ) ) and
            not( MiniPet:DoCustomEventWithReturn( "IsCompanionSummonInCooldown" ) ) ) then
            self.counter = self.counter + 1
            if( self.counter > 30 ) then
                if( MiniPet:DoCustomEventWithReturn( "SummonCompanion" ) ) then
                    self.delaySummon = false
                    self.counter = 0
                else
                    self.counter = 0
                end--if
            end--if
        else
            self.counter = 0
        end--if
    else
        self.counter = 0
        self.delaySummon = not( MiniPet:DoCustomEventWithReturn( "HasDismissableCompanionOut" ) )
    end--if
end--AutoSummon:OnUpdate( MiniPet )

function AutoSummon.IsInPvPZone()
    local _, itype = IsInInstance()
    local pvpType, isFFA = GetZonePVPInfo()

    return isFFA or ( itype == "pvp" ) or ( itype == "arena" ) or ( pvpType == "combat" )
end--AutoSummon.IsInPvPZone()

function AutoSummon.IsStealthed()
    return IsStealthed() or
        ( SpecialEvents_Aura:UnitHasBuff( "player", 66 ) ) or -- Invisiblity
        ( SpecialEvents_Aura:UnitHasBuff( "player", 51753 ) ) or -- Camouflage
        ( SpecialEvents_Aura:UnitHasBuff( "player", 51755 ) ) -- Camouflage
end--

LibStub( "LibLordFarlander-Randomizer-2.0" ):RegisterModuleToHandler( "MiniPet", moduleName, AutoSummon, "Companions" )
